/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
    tvp5150.h - definition for tvp5150 inputs

    Copyright (C) 2006 Hans Verkuil (hverkuil@kernel.org)

*/

#ifndef _DT_BINDINGS_MEDIA_TVP5150_H
#define _DT_BINDINGS_MEDIA_TVP5150_H

/* TVP5150 HW inputs */
#define TVP5150_COMPOSITE0 0
#define TVP5150_COMPOSITE1 1
#define TVP5150_SVIDEO     2

/* TVP5150 HW outputs */
#define TVP5150_NORMAL       0
#define TVP5150_BLACK_SCREEN 1

#endif /* _DT_BINDINGS_MEDIA_TVP5150_H */
