/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2023, Linaro Ltd.
 */

#ifndef _DT_BINDINGS_CLK_QCOM_LPASSCC_SM6115_H
#define _DT_BINDINGS_CLK_QCOM_LPASSCC_SM6115_H

/* LPASS CC */
#define LPASS_SWR_TX_CONFIG_CGCR		0

/* LPASS_AUDIO CC */
#define LPASS_AUDIO_SWR_RX_CGCR			0

#endif
