/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_POWER_MT6735_POWER_CONTROLLER_H
#define _DT_BINDINGS_POWER_MT6735_POWER_CONTROLLER_H

#define MT6735_POWER_DOMAIN_MD1		0
#define MT6735_POWER_DOMAIN_CONN	1
#define MT6735_POWER_DOMAIN_DIS		2
#define MT6735_POWER_DOMAIN_MFG		3
#define MT6735_POWER_DOMAIN_ISP		4
#define MT6735_POWER_DOMAIN_VDE		5
#define MT6735_POWER_DOMAIN_VEN		6

#endif
