/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_CLK_MT6735_VENCSYS_H
#define _DT_BINDINGS_CLK_MT6735_VENCSYS_H

#define CLK_VENC_SMI_LARB3			0
#define CLK_VENC_VENC			1
#define CLK_VENC_JPGENC			2
#define CLK_VENC_JPGDEC			3

#endif /* _DT_BINDINGS_CLK_MT6735_VENCSYS_H */
