/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2015 Google, Inc.
 */

#ifndef _DT_BINDINGS_PHY_PISTACHIO
#define _DT_BINDINGS_PHY_PISTACHIO

#define REFCLK_XO_CRYSTAL	0x0
#define REFCLK_X0_EXT_CLK	0x1
#define REFCLK_CLK_CORE		0x2

#endif /* _DT_BINDINGS_PHY_PISTACHIO */
