/* SPDX-License-Identifier: GPL-2.0 */

#ifndef __DT_BINDINGS_TI_SCI_PM_DOMAIN_H
#define __DT_BINDINGS_TI_SCI_PM_DOMAIN_H

#define TI_SCI_PD_EXCLUSIVE	1
#define TI_SCI_PD_SHARED	0

#endif /* __DT_BINDINGS_TI_SCI_PM_DOMAIN_H */
