/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (C) 2023 Sophgo Technology Inc. All rights reserved.
 */

#ifndef __DT_BINDINGS_SOPHGO_SG2042_PLL_H__
#define __DT_BINDINGS_SOPHGO_SG2042_PLL_H__

#define MPLL_CLK			0
#define FPLL_CLK			1
#define DPLL0_CLK			2
#define DPLL1_CLK			3

#endif /* __DT_BINDINGS_SOPHGO_SG2042_PLL_H__ */
